import logging
import os
import os.path
import subprocess

import drvn.installer._utils as utils


def install(is_install_drvn_configs):
    logging.info("Installing duplicity ...")
    _install_prerequisites()
    _install_duplicity()
    if is_install_drvn_configs:
        _install_drvn_configs()


def _install_prerequisites():
    apt_dependencies = [
        "git",
        "build-essential",
        "libssl-dev",
        "libffi-dev",
        "apt-utils",
        "python3-dev",
        "python3-pip",
        "python3-distutils-extra",
        "librsync-dev",
    ]
    utils.try_cmd(f"sudo apt-get install -y {' '.join(apt_dependencies)}")

    pip_dependencies = ["future>=0.18.2", "fasteners>=0.15"]
    utils.try_cmd(f"python3 -m pip install --user {' '.join(pip_dependencies)}")


def _install_duplicity():
    if not os.path.exists(os.path.expanduser("~/duplicity")):
        utils.try_cmd(
            "git clone https://gitlab.com/duplicity/duplicity.git",
            cwd=os.path.expanduser("~"),
        )
    utils.try_cmd(
        "sudo python3 setup.py install", cwd=os.path.expanduser("~/duplicity"),
    )


def _install_drvn_configs():
    pass



# pylint: disable=no-self-use, protected-access
import drvn.installer.installers.tmux_installer as installer

import pytest
from unittest.mock import MagicMock, patch


class TestCalculateUrlToLatestReleaseTarball:
    def test_normal(self):
        calculated_url = installer._calculate_url_to_latest_release_tarball("2.9a")
        assert (
            calculated_url
            == "https://github.com/tmux/tmux/releases/download/2.9a/tmux-2.9a.tar.gz"
        )


@patch("drvn.installer._utils.open_url")
class TestFindUrlToLatestRelease:
    def test_normal(self, open_url):
        open_url.return_value = '{"tag_name":"3.0a"}'
        assert "https" in installer._find_url_to_latest_release()



# pylint: disable=no-self-use, protected-access
import drvn.installer.installer as installer

import pytest


class TestGetInstallableSoftware:
    def test_returns_a_list(self):
        installable_software = installer.get_installable_software()

        assert type(installable_software) is list



import logging

from drvn.installer.installers import (
    ultimate_vim_installer,
    tmux_installer,
    duplicity_installer,
)


def install(software, is_install_drvn_configs):
    if software not in get_installable_software():
        raise RuntimeError(
            f"Installing '{software}' is not supported by this installer."
        )

    install_function = _get_install_function(software)
    install_function(is_install_drvn_configs)

    logging.info("Install successful")


def get_installable_software():
    installable_software = [
        software
        for software, install_function in _SOFTWARE_TO_INSTALL_FUNCTION.items()
    ]
    return installable_software


def _get_install_function(software):
    return _SOFTWARE_TO_INSTALL_FUNCTION[software]


_SOFTWARE_TO_INSTALL_FUNCTION = {
    "tmux": tmux_installer.install,
    "ultimate_vim": ultimate_vim_installer.install,
    "duplicity": duplicity_installer.install,
}



import logging
import os
import os.path
import argparse
import json
import subprocess
from pathlib import Path

import drvn.installer._utils as utils
import drvn.installer._dotfile_repository as dotfile_repository


def install(is_install_drvn_configs):
    logging.info("Installing tmux ...")
    _install_prerequisites()
    _install_tmux()
    if is_install_drvn_configs:
        _install_drvn_configs()


def _install_prerequisites():
    apt_dependencies = ["libevent-dev", "libncurses5-dev", "curl"]
    utils.try_cmd(f"sudo apt-get install -y {' '.join(apt_dependencies)}")


def _install_tmux():
    latest_release_url = _find_url_to_latest_release()
    _install_tmux_from_url(latest_release_url)


def _install_tmux_from_url(tarball_url):
    version_tag = _find_version_tag_of_latest_release()
    tarball_file_name = f"tmux-{version_tag}.tar.gz"

    tarball_download_location = f"{os.environ['HOME']}/{tarball_file_name}"

    utils.try_cmd(
        f"cd ~ && "
        + f"curl -L -o {tarball_download_location} {tarball_url} && "
        + f"tar -xzf {tarball_file_name} && "
        + f"cd {tarball_file_name.replace('.tar.gz', '')} && "
        + f"./configure && "
        + f"make && "
        + f"sudo make install",
    )


def _install_drvn_configs():
    if not dotfile_repository.is_cloned():
        dotfile_repository.clone()
    _symlink_configs_to_dotfile_repository()


def _symlink_configs_to_dotfile_repository():
    dotfile_relative_path = Path(".tmux.conf")
    real_path = dotfile_repository.get_path(dotfile_relative_path)
    symlink_path = Path("~").expanduser() / dotfile_relative_path
    utils.cmd(f"rm {symlink_path}")
    utils.try_cmd(f"ln -s '{real_path}' '{symlink_path}'")


def _find_url_to_latest_release():
    version_tag = _find_version_tag_of_latest_release()
    url_to_latest_release_tarball = _calculate_url_to_latest_release_tarball(
        version_tag
    )
    return url_to_latest_release_tarball


def _get_project_root_dir():
    dn = os.path.dirname
    project_root_dir = dn(os.path.abspath(__file__))
    return project_root_dir


def _find_version_tag_of_latest_release():
    response = utils.open_url(
        "https://api.github.com/repos/tmux/tmux/releases/latest"
    )
    response_dict = json.loads(response)
    version_tag = response_dict["tag_name"]
    return version_tag


def _calculate_url_to_latest_release_tarball(version_tag):
    url_to_latest_release_tarball = (
        f"https://github.com/tmux/tmux/releases/download"
        f"/{version_tag}/tmux-{version_tag}.tar.gz"
    )
    return url_to_latest_release_tarball
